// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math';
import 'dart:typed_data';

import 'package:crypto/crypto.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:nim_core/nim_core.dart';
import 'package:path_provider/path_provider.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  // For Publish Use
  // static const appKey = 'Your_App_Key';
  // static const account = 'Account_ID';
  // static const token = 'Account_Token';
  // static const friendAccount = 'Friend_Account_ID';
  // static const chatroomId = '123456789';

  // Inner Test AppKey
  static const appKey = '45c6af3c98409b18a84451215d0bdd6e';
  static const chatroomId = '3001';

  // Inner Test Configuration 1
  static const account = '260974291521792';
  static const token = '5b8b5bca-c4b4-4d60-884f-42adc6425fa2';
  static const friendAccount = '263248940175616';

  // Inner Test Configuration 2
  // static const account = 'lcd123456';
  // static const token = '1f57ef61a99370211f126e9dd8ba88bd';
  // static const friendAccount = 'jinjie03';

  final subsriptions = <StreamSubscription>[];

  Uint8List? _deviceToken;

  void updateAPNsToken() {
    if (NimCore.instance.isInitialized &&
        Platform.isIOS &&
        _deviceToken != null) {
      NimCore.instance.settingsService.updateAPNSToken(_deviceToken!, null);
    }
  }

  @override
  void initState() {
    super.initState();

    MethodChannel('com.netease.NIM.demo/settings')
        .setMethodCallHandler((call) async {
      if (call.method == 'updateAPNsToken') {
        print('update APNs token');
        _deviceToken = call.arguments as Uint8List;
      }
      return null;
    });

    subsriptions.add(NimCore.instance.authService.authStatus.listen((event) {
      print('AuthService##auth status event: ${event.status.name}');
      if (event.status == NIMAuthStatus.loggedIn) {
        //_testTeam();
      }
    }));

    // subsriptions
    //     .add(NimCore.instance.messageService.onMessage.listen((messages) {
    //   messages.forEach((message) {
    //     print(
    //         'MessageService##receive message: ${message.fromNickname} says ${message.content}');
    //   });
    // }));

    // subsriptions
    //     .add(NimCore.instance.messageService.onMessagePinNotify.listen((event) {
    //   print('MessageService##receive message pin: $event ');
    // }));

    // subsriptions.add(
    //     NimCore.instance.messageService.onStickTopSessionAdd.listen((session) {
    //   print('MessageService##onStickTopSessionAdd: $session ');
    // }));

    // subsriptions.add(NimCore.instance.messageService.onStickTopSessionUpdate
    //     .listen((session) {
    //   print('MessageService##onStickTopSessionUpdate: $session ');
    // }));

    // subsriptions.add(NimCore.instance.messageService.onStickTopSessionRemove
    //     .listen((session) {
    //   print('MessageService##onStickTopSessionRemove: $session ');
    // }));

    // subsriptions.add(
    //     NimCore.instance.messageService.onSyncStickTopSession.listen((session) {
    //   print('MessageService##onSyncStickTopSession: ${session.join('##')} ');
    // }));

    // subsriptions.add(
    //     NimCore.instance.messageService.onQuickCommentAdd.listen((comment) {
    //   print('MessageService##onQuickCommentAdd: $comment');
    // }));

    // subsriptions.add(
    //     NimCore.instance.messageService.onQuickCommentRemove.listen((comment) {
    //   print('MessageService##onQuickCommentRemove: $comment');
    // }));

    // subsriptions.add(
    //     NimCore.instance.messageService.onMySessionUpdate.listen((session) {
    //   print('MessageService##onMySessionUpdate: $session');
    // }));

    // subsriptions
    //     .add(NimCore.instance.messageService.onSessionDelete.listen((session) {
    //   print('MessageService##onSessionDelete: $session');
    // }));

    // subsriptions
    //     .add(NimCore.instance.messageService.onSessionUpdate.listen((session) {
    //   print('MessageService##onSessionUpdate: $session');
    // }));

    _doInitializeSDK();
  }

  void _doInitializeSDK() async {
    late NIMWINDOWSSDKOptions options;
    options = NIMWINDOWSSDKOptions(
      appKey: appKey,
    );

    NimCore.instance.initialize(options).then((value) async {
      NimCore.instance.authService.dynamicTokenProvider = (account) async {
        print('AuthService##getDynamicToken: $account');
        final currentTime = DateTime.now().millisecondsSinceEpoch;
        final ttl = 600;
        final secret = '';
        final signature =
            sha1.convert(utf8.encode('$appKey$account$currentTime$ttl$secret'));
        return base64.encode(utf8.encode(
            '{"signature":"$signature","curTime":$currentTime,"ttl":$ttl}'));
      };
      final loginResult = await NimCore.instance.authService.login(NIMLoginInfo(
        account: account,
        token: token,
        // authType: NIMAuthType.authTypeDynamic,
      ));

      print('login result: $loginResult');
    });
  }

  void _testTeam() async {
    final ret1 = await NimCore.instance.teamService.createTeam(
      createTeamOptions: NIMCreateTeamOptions(
        name: "测试群聊",
        avatarUrl:
            'https://t7.baidu.com/it/u=2168645659,3174029352&fm=193&f=GIF',
        introduce: 'introduce',
        announcement: 'announcement',
        extension: 'extension',
        postscript: 'postscript',
        verifyType: NIMVerifyTypeEnum.free,
        inviteMode: NIMTeamInviteModeEnum.all,
        beInviteMode: NIMTeamBeInviteModeEnum.noAuth,
        updateInfoMode: NIMTeamUpdateModeEnum.all,
        extensionUpdateMode: NIMTeamExtensionUpdateModeEnum.all,
        teamType: NIMTeamTypeEnum.advanced,
      ),
      members: ['jinjie00'],
    );
    print('_testTeam createTeam ${ret1.data!.team!.id!}');
    String teamId = ret1.data!.team!.id!;
    final ret2 = await NimCore.instance.teamService.searchTeam(teamId);
    print(
        '_testTeam searchTeam ${ret2.data!.id!}, ${ret2.data!.name!}, ${ret2.data!.createTime}');
    final ret3 = await NimCore.instance.teamService.queryTeamList();
    print('_testTeam queryTeamList ${ret3.data!.length}');
    final ret4 = await NimCore.instance.teamService.queryTeam(teamId);
    print(
        '_testTeam queryTeam ${ret4.data!.id!}, ${ret4.data!.name!}, ${ret4.data!.createTime}');
    final ret5 = await NimCore.instance.teamService.searchTeamIdByName('测试群聊');
    print('_testTeam searchTeamIdByName ${ret5.data}');
    final ret6 =
        await NimCore.instance.teamService.searchTeamsByKeyword('测试群聊');
    print('_testTeam searchTeamsByKeyword ${ret6.data}');
    final ret7 = await NimCore.instance.teamService.addMembersEx(
        teamId: teamId,
        accounts: [friendAccount],
        msg: '邀请入群',
        customInfo: '自定义消息');
    print('_testTeam addMembersEx ${ret7.code} ${ret7.data}');
    final ret8 = await NimCore.instance.teamService.queryMemberList(teamId);
    print('_testTeam queryMemberList ${ret8.data!.length}');
    final ret9 =
        await NimCore.instance.teamService.queryTeamMember(teamId, 'jinjie00');
    print('_testTeam queryTeamMember ${ret9.data?.account}');
    final ret10 = await NimCore.instance.teamService
        .updateMemberNick(teamId, 'jinjie00', 'nick');
    print('_testTeam updateMemberNick ${ret10.code}');
    final ret11 =
        await NimCore.instance.teamService.addManagers(teamId, ['jinjie00']);
    print('_testTeam addManagers ${ret11.code}');
    final ret12 =
        await NimCore.instance.teamService.removeManagers(teamId, ['jinjie00']);
    print('_testTeam removeManagers ${ret12.code}');
    final ret13 =
        await NimCore.instance.teamService.muteAllTeamMember(teamId, true);
    print('_testTeam muteAllTeamMember ${ret13.code}');
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setName('更新群名称');
    final ret14 =
        await NimCore.instance.teamService.updateTeamFields(teamId, request);
    print('_testTeam updateTeamFields ${ret14.code}');
    final ret100 = await NimCore.instance.teamService.searchTeam(teamId);
    print(
        '_testTeam searchTeam ${ret100.data!.id!}, ${ret100.data!.name!}, ${ret100.data!.createTime}');
    final ret101 = await NimCore.instance.teamService.searchTeam(teamId);
    print(
        '_testTeam searchTeam ${ret101.data!.id!}, ${ret101.data!.name!}, ${ret101.data!.createTime}');
    final ret102 = await NimCore.instance.teamService
        .muteTeamMember(teamId, 'jinjie00', true);
    print('_testTeam muteTeamMember ${ret102.code}');
    final ret103 = await NimCore.instance.teamService
        .muteTeamMember(teamId, 'jinjie00', false);
    print('_testTeam unmuteTeamMember ${ret102.code}');
    final ret16 = await NimCore.instance.teamService
        .muteTeam(teamId, NIMTeamMessageNotifyTypeEnum.all);
    print('_testTeam muteTeam ${ret16.code}');
    final ret17 = await NimCore.instance.teamService
        .transferTeam(teamId, 'jinjie00', false);
    print('_testTeam transferTeam ${ret17.code}');
    final ret18 = await NimCore.instance.teamService.quitTeam(teamId);
    print('_testTeam quitTeam ${ret18.code}');
    final ret19 = await NimCore.instance.teamService.createTeam(
        createTeamOptions: NIMCreateTeamOptions(
            name: "测试群聊", teamType: NIMTeamTypeEnum.normal),
        members: ['jinjie00']);
    String teamId1 = ret19.data!.team!.id!;
    final ret20 = await NimCore.instance.teamService.dismissTeam(teamId1);
    print('_testTeam dismissTeam ${ret20.code}');
  }

  @override
  void dispose() {
    subsriptions.forEach((subsription) {
      subsription.cancel();
    });
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(),
      ),
    );
  }
}
